package sample;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import sample.dao.UserDao;
import sample.entity.User;

public class DaoCallServiceImpl implements DaoCallService {

    private final static Logger logger = LoggerFactory.getLogger(DaoCallServiceImpl.class);

    // Spring̐ݒDI
    private EntityManagerFactory entityManagerFactory;

    // Spring̐ݒDI
    private UserDao userDao;

    public void setEntityManagerFactory(EntityManagerFactory entityManagerFactory) {
        this.entityManagerFactory = entityManagerFactory;
    }

    public void setUserDao(UserDao userDao) {
        this.userDao = userDao;
    }

    public void execute() {

        logger.info("Sample code log ---------- daoCallService.execute() Start");

        //DB Sessionm SpringDIĂꂽEntityManagerFactoryentityManager
        //SpringDIEntityManagerFactory؂ւ鎖DB̐ڑύX\
        EntityManager entityManager = entityManagerFactory.createEntityManager();

        logger.info("Sample code log ---------- transaction start");
        EntityTransaction entityTransaction = entityManager.getTransaction();
        entityTransaction.begin();

        try {
            logger.info("Sample code log ---------- DB̃f[^Sď");
            initDB(entityManager);

            logger.info("Sample code log ---------- User5f[^");
            insertUser(entityManager);

            logger.info("Sample code log ---------- S");
            selectAll(entityManager);

            logger.info("Sample code log ---------- Id:3 update");
            updateId3(entityManager);

            logger.info("Sample code log ---------- Select JPQL userName:2");
            selectJPQL(entityManager);

            logger.info("Sample code log ---------- Select namedQL userName:4");
            selectNamedQL(entityManager);

            logger.info("Sample code log ---------- Select NativeQL userName:5");
            selectNativQL(entityManager);

            logger.info("Sample code log ---------- transaction commit");
            entityTransaction.commit();

        } catch (Exception e) {

            logger.error("Sample code log ---------- Exception:" + e.getMessage());

            entityTransaction.rollback();
            logger.info("Sample code log ---------- transaction rollback");

        } finally {

            entityManager.close();
            entityManagerFactory.close();
        }

        logger.info("End---------------------------");
    }

    private void initDB(EntityManager entityManager) {

        logger.info("Sample code log ---------- initDB Start");

        List<User> users = userDao.selectAll(entityManager);

        for (User user : users) {
            userDao.remove(entityManager, user.getId());
        }

        logger.info("Sample code log ---------- initDB End");
    }

    private void insertUser(EntityManager entityManager) {

        logger.info("Sample code log ---------- insertUser Start");

        int whileCount = 0;

        while (whileCount < 5) {
            int id = whileCount + 1;
            User user = new User(id, "userName:" + id, "password:" + id);

            userDao.insert(entityManager, user);

            whileCount++;
        }

        logger.info("Sample code log ---------- insertUser End");
    }

    private void selectAll(EntityManager entityManager) {

        logger.info("Sample code log ---------- selectAll Start");

        List<User> users = userDao.selectAll(entityManager);

        for (User user : users) {
            logger.info("Sample code log ---------- selectAll Result:" + user.toString());
        }

        logger.info("Sample code log ---------- selectAll End");
    }

    private void updateId3(EntityManager entityManager) {

        logger.info("Sample code log ---------- updateId3 Start");

        User user = new User(3, "Change3", "Change3Pass");
        userDao.update(entityManager, user);
        user = userDao.selectId(entityManager, 3);

        logger.info("Sample code log ---------- updateId3 Select Result:" + user.toString());
        logger.info("Sample code log ---------- updateId3 End");
    }

    private void selectJPQL(EntityManager entityManager) {

        logger.info("Sample code log ---------- selectJPQL Start");

        List<User> users = userDao.selectJPQLUserName(entityManager, "userName:2");
        User user = users.get(0);

        logger.info("Sample code log ---------- selectJPQL Select Result:" + user.toString());
        logger.info("Sample code log ---------- selectJPQL End");
    }

    private void selectNamedQL(EntityManager entityManager) {

        logger.info("Sample code log ---------- selectNamedQL Start");

        List<User> users = userDao.selectNamedQLUserName(entityManager, "userName:4");
        User user = users.get(0);

        logger.info("Sample code log ---------- selectNamedQL Select Result:" + user.toString());
        logger.info("Sample code log ---------- selectNamedQL End");
    }

    private void selectNativQL(EntityManager entityManager) {

        logger.info("Sample code log ---------- selectNativQL Start");

        List<User> users = userDao.selectNativeQLUserName(entityManager, "userName:5");
        User user = users.get(0);

        logger.info("Sample code log ---------- selectNativQL Select Result:" + user.toString());
        logger.info("Sample code log ---------- selectNativQL End");
    }
}
