package sample.dao;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import sample.entity.User;

public class UserDaoImpl implements UserDao {

    final Logger logger = LoggerFactory.getLogger(UserDaoImpl.class);

    //SR[h̎擾
    public List<User> selectAll(EntityManager entityManager) {

        Query query = entityManager.createQuery("SELECT user FROM User as user order by user.id");

        return query.getResultList();
    }

    // PKłSELECT
    public User selectId(EntityManager entityManager, Integer id) {

        return entityManager.find(User.class, id);
    }

    // JPQLłSELECT
    public List<User> selectJPQLUserName(EntityManager entityManager, String userName) {

        Query query =
                entityManager.createQuery("SELECT user FROM User as user " + "WHERE user.userName = :userName ")
                    .setParameter("userName", userName);

        return query.getResultList();
    }

    // Named QueryłSELECT
    public List<User> selectNamedQLUserName(EntityManager entityManager, String userName) {

        Query query = entityManager.createNamedQuery("selectNamed").setParameter("userName", userName);

        return query.getResultList();
    }

    // Native QueryłSELECT
    public List<User> selectNativeQLUserName(EntityManager entityManager, String userName) {

        // SQL̂܂܂Ŏs\ p[^͐lŵ
        Query query =
                entityManager.createNativeQuery("SELECT id, userName, password FROM user WHERE username = ?1 ",
                    User.class).setParameter(1, userName);

        return query.getResultList();
    }

    // insert
    public void insert(EntityManager entityManager, User user) {

        // DB։i
        entityManager.persist(user);
        logger.info("Sample code log ---------- insert execute(" + user.toString() + " )");
    }

    // update
    public void update(EntityManager entityManager, User user) {

        // L[Ō
        Integer id = user.getId();
        User dbUser = entityManager.find(User.class, id);
        logger.info("Sample code log ---------- update execute before(" + dbUser.toString() + " )");

        // ̓eŃR[hXV
        dbUser.setUserName(user.getUserName());
        dbUser.setPassword(user.getPassword());
        logger.info("Sample code log ---------- update execute after(" + dbUser.toString() + " )");
    }

    // delete
    public void remove(EntityManager entityManager, Integer id) {

        // idŌč폜
        User user = entityManager.find(User.class, id);
        entityManager.remove(user);

        logger.info("Sample code log ---------- delete execute (" + user.toString() + " )");
    }
}
