package sample;

import java.util.List;

import org.hibernate.ScrollableResults;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.classic.Session;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import sample.dao.UserDao;
import sample.entity.User;

public class DaoCallServiceImpl implements DaoCallService {

    private Logger logger = LoggerFactory.getLogger(DaoCallServiceImpl.class);

    // Spring̐ݒDI
    private SessionFactory sessionFactory;

    // Spring̐ݒDI
    private UserDao userDao;

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public void setUserDao(UserDao userDao) {
        this.userDao = userDao;
    }

    public void execute() {

        logger.info("Sample code log ---------- daoCallService.execute() Start");

        //DB Sessionm SpringDIĂꂽSessionFactorySession
        //SpringDISessionFactory؂ւ鎖DB̐ڑύX\
        Session session = (Session) sessionFactory.openSession();

        logger.info("Sample code log ---------- transaction start");
        Transaction tx = session.beginTransaction();

        try {
            logger.info("Sample code log ---------- DB̃f[^Sď");
            initDB(session);

            logger.info("Sample code log ---------- User5f[^");
            insertUser(session);

            logger.info("Sample code log ---------- S");
            selectAll(session);

            logger.info("Sample code log ---------- Id:3 update");
            updateId3(session);

            logger.info("Sample code log ---------- Select HQL userName:2");
            selectHQL(session);

            logger.info("Sample code log ---------- Select Criteria userName:4");
            selectCriteria(session);

            logger.info("Sample code log ---------- Select NativeQL userName:5");
            selectNativQL(session);

            logger.info("Sample code log ---------- transaction commit");
            tx.commit();

        } catch (Exception e) {

            logger.error("Sample code log ---------- Exception:" + e.getMessage());

            tx.rollback();
            logger.info("Sample code log ---------- transaction rollback");

        } finally {

            session.close();
            sessionFactory.close();
        }

        logger.info("End---------------------------");
    }

    private void initDB(Session session) {

        logger.info("Sample code log ---------- initDB Start");

        ScrollableResults scrollableResults = userDao.selectAll(session);

        while (scrollableResults.next()) {
            User listUser = (User) scrollableResults.get(0);
            userDao.delete(session, listUser.getId());
        }

        logger.info("Sample code log ---------- initDB End");
    }

    private void insertUser(Session session) {

        logger.info("Sample code log ---------- insertUser Start");

        int whileCount = 0;

        while (whileCount < 5) {
            int id = whileCount + 1;
            User user = new User(id, "userName:" + id, "password:" + id);

            userDao.insert(session, user);

            whileCount++;
        }

        logger.info("Sample code log ---------- insertUser End");
    }

    private void selectAll(Session session) {

        logger.info("Sample code log ---------- selectAll Start");

        ScrollableResults scrollableResults = userDao.selectAll(session);

        while (scrollableResults.next()) {

            User user = (User) scrollableResults.get(0);
            logger.info(user.toString());
        }

        logger.info("Sample code log ---------- selectAll End");
    }

    private void updateId3(Session session) {

        logger.info("Sample code log ---------- updateId3 Start");

        User user = new User(3, "Change3", "Change3Pass");
        userDao.update(session, user);
        user = userDao.selectId(session, 3);

        logger.info("Sample code log ---------- updateId3 Select Result:" + user.toString());
        logger.info("Sample code log ---------- updateId3 End");
    }

    private void selectHQL(Session session) {

        logger.info("Sample code log ---------- selectHQL Start");

        ScrollableResults scrollableResults = userDao.selectHQLUserName(session, "userName:2");
        scrollableResults.next();
        User user = (User) scrollableResults.get(0);

        logger.info("Sample code log ---------- selectHQL Select Result:" + user.toString());
        logger.info("Sample code log ---------- selectHQL End");
    }

    private void selectCriteria(Session session) {

        logger.info("Sample code log ---------- selectCriteria Start");

        ScrollableResults scrollableResults = userDao.selectCriteriaUserName(session, "userName:4");
        scrollableResults.next();
        User user = (User) scrollableResults.get(0);

        logger.info("Sample code log ---------- selectCriteria Select Result:" + user.toString());
        logger.info("Sample code log ---------- selectCriteria End");
    }

    private void selectNativQL(Session session) {

        logger.info("Sample code log ---------- selectNativQL Start");

        List<Object[]> objs = userDao.selectNativeQLUserName(session, "userName:5");
        Object[] obj = objs.get(0);
        User user = new User(Integer.valueOf(obj[0].toString()), obj[1].toString(), obj[2].toString());

        logger.info("Sample code log ---------- selectNativQL Select Result:" + user.toString());
        logger.info("Sample code log ---------- selectNativQL End");
    }
}
