package sample.dao;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import sample.entity.User;

public class UserDaoImpl implements UserDao {

    final Logger logger = LoggerFactory.getLogger(UserDaoImpl.class);

    //SR[h̎擾
    public ScrollableResults selectAll(Session session) {

        return session.createQuery("SELECT user FROM User as user order by user.id").scroll();
    }

    // PKłSELECT
    public User selectId(Session session, Integer id) {
        Criteria criteria = session.createCriteria(User.class);
        criteria.add(Restrictions.eq("id", id));

        List<User> users = criteria.list();

        if (users != null) {
            return users.get(0);
        } else {
            return null;
        }
    }

    // HQLłSELECT
    public ScrollableResults selectHQLUserName(Session session, String userName) {

        return session.createQuery("SELECT user FROM User as user WHERE user.userName = :userName").setString(
            "userName", userName).scroll();
    }

    // CriteriałSELECT
    public ScrollableResults selectCriteriaUserName(Session session, String userName) {

        Criteria criteria = session.createCriteria(User.class);
        criteria.add(Restrictions.eq("userName", userName));

        return criteria.scroll();
    }

    // Native QueryłSELECT
    public List<Object[]> selectNativeQLUserName(Session session, String userName) {

        return session.createSQLQuery("SELECT id, userName, password FROM User WHERE username = :userName ")
            .setParameter("userName", userName)
            .list();
    }

    // insert
    public void insert(Session session, User user) {

        session.persist(user);
        logger.info("Sample code log ---------- insert execute(" + user.toString() + " )");
    }

    // update
    public void update(Session session, User user) {

        User dbUser = selectId(session, user.getId());

        if (user != null) {

            logger.info("Sample code log ---------- update execute before(" + dbUser.toString() + " )");

            dbUser.setUserName(user.getUserName());
            dbUser.setPassword(user.getPassword());

            logger.info("Sample code log ---------- update execute after(" + user.toString() + " )");
        } else {
            logger.info("Sample code log ---------- no update");
        }
    }

    // delete
    public void delete(Session session, Integer id) {

        User user = selectId(session, id);

        if (user != null) {

            session.delete(user);

            logger.info("Sample code log ---------- delete execute (" + user.toString() + " )");
        } else {
            logger.info("Sample code log ---------- no delete");
        }
    }
}
